import QtQuick

QtObject {

  readonly property int  osMode:                  1

  // controller preferences
  property bool mixerFXSelector:                  false
  property bool prioritizeFXSelection:            false

  readonly property bool s12Mode:                 false //Set to true if you are using S8 + 2x D2

  property bool fineMasterTempoAdjust:            true
  property bool fineDeckTempoAdjust:              true

  // global preferences
  property bool camelotKey:                       true
  property int  phraseLength:                     4

  // browser preferences
  readonly property bool displayMatchGuides:      true
  readonly property bool displayMoreItems:        true

  // deck preferences
  property bool displayAlbumCover:                true
  property bool displayHotCueBar:                 true
  property bool displayDeckIndicators:            true
  property bool displayPhaseMeter:                true

  property int spectrumWaveformColors:            7 // Change for Spectrum Waveform Colors Themes
                                                    // 1 - KOKERNUTZ, 
                                                    // 2 - NEXUS
                                                    // 3 - PRIME
                                                    // 4 - Denon SC5000 / SC6000
                                                    // 5 - Pioneer CDJ 2000
                                                    // 6 - Pioneer CDJ 3000
                                                    // 7 - NUMARK
                                                    // 8 - X RAY
                                                    // 9 - Infrared
                                                    // 10 - Ultraviolet
                                                    // 11 - Spectrum-like colors

  property variant mixerFXNames:         ["FLTR", "MFX 1", "MFX 2", "MFX 3", "MFX 4"] // do not change FLTR

  // deck header text (use -1 to turn off)
  readonly property int topLeftText:      0
  readonly property int topCenterText:    14
  readonly property int topRightText:     12

  readonly property int middleLeftText:   1
  readonly property int middleCenterText: 31
  readonly property int middleRightText:  24

  readonly property int bottomLeftText:   19
  readonly property int bottomCenterText: 15
  readonly property int bottomRightText:  28

  // options:
  //
  // 0:  "title",          1: "artist",       2:  "release", 
  // 3:  "mix",            4: "label",        5:  "catNo", 
  // 6:  "genre",          7: "trackLength",  8:  "bitrate", 
  // 9:  "bpmTrack",      10: "gain",        11: "elapsedTime", 
  // 12: "remainingTime", 13: "beats",       14: "beatsToCue", 
  // 15: "bpm",           16: "tempo",       17: "key", 
  // 18: "keyText",       19: "comment",     20: "comment2",
  // 21: "remixer",       22: "pitchRange",  23: "bpmStable", 
  // 24: "tempoStable",   25: "sync",        26: "off", 
  // 27: "off",           28: "bpmTrack"     29: "remixBeats"
  // 30: "remixQuantize", 31: "keyDisplay"

  // UI preferences
  property string fontName: "Consolas"
  property string fontMediumName: "TRAKTORFREON"

  property string fontDeckHeaderNumber: "Consolas"
  property string fontDeckHeaderString: "TRAKTORFREON"

  property string fontDeckFooterNumber: "Consolas"
  property string fontDeckFooterString: "TRAKTORFREON"

  // A) DEFAULT
  // "Pragmatica"
  // "Pragmatica MediumTT"

  // B) GAMER
  // "Consolas"
  // "TRAKTORFREON"

  // C) ROBOTO
  // "Roboto"
  // "Roboto Medium"

  // X) OTHER FONTS SUPPORTED
  // "Consolas" // Ok
  // "Consolas Bold"
  // "Roboto"
  // "Roboto Light"
  // "Roboto Medium"
  // "Roboto Regular"
  // "TRAKTORFREON" // Ok
  // "UpheavalX1" // Ok
  // "ProggyCleanTT" // Require install
  // "ProggyVector" // Require install
  // "Webdings"

  //--------------------------------------------------------------------------------------------------------------------
  // LOG SYSTEM
  //--------------------------------------------------------------------------------------------------------------------

  property bool debug: false
  property string debugResult: ""

  function log(message) {
    var count = (debugResult.match(/<br>/g) || []).length;

    if (count > 4) {
      // break the textblock into an array of lines
      var lines = debugResult.split('<br>');
      // remove one line, starting at the first position
      lines.splice(0,1);
      // join the array back into a single string
      debugResult = lines.join('<br>');
    }

    debugResult +=  "<br>" + message;
  }

  //--------------------------------------------------------------------------------------------------------------------
  // S4 MK3
  //--------------------------------------------------------------------------------------------------------------------

  //set to false to disable the adjacent key colouring and return to all keys coloured
  readonly property bool    adjacentKeys: true
   
  //change to false to disable FX overlays
  readonly property bool fxOverlays: true
  
  //change to false to diable the browser view
  readonly property bool enableBrowser: true
  readonly property bool hideBPM: false
  readonly property bool hideKey: false
  readonly property bool showArtistColumn: false
  readonly property bool showTrackTitleColumn: true
  readonly property int browserFontSize: 20 // origine : 15
  readonly property bool raiseBrowserFooter: true  // origine : false
  
  //change the values below to determine the bpm text colour in the browser, the number values represent the percentage difference of the master tempo and the selected song
  readonly property bool bpmBrowserTextColor: true
  readonly property int browserBpmGreen: 4 // origine : 3
  readonly property int browserBpmRed: 200 // origine : 12
  
  //change to false to disable the browser view displaying artist data whilst holding shift
  readonly property bool browserShift: true
  
  //change to false to disable moving waveforms, WITH THIS ENABLED THE BEATS BOX IS COMBINED WITH THE TIME BOX. USE enableBeatsBox TO ALTERNATE THE VALUE OF THE TIME/BEATS BOX
  readonly property bool fullWaveforms: true
  readonly property bool hideStripe: false
  
  //change to false to disable the switching of stripes when holding shift
  readonly property bool shiftStripe: false
  
  //change to true to show the master deck's stripe when holding shift (shiftStripe must be set to true)
  readonly property bool shiftStripeMaster: false
  
  //zoom level for the moving waveform, lower value means the waveform is more zoomed
  readonly property int fullWaveformZoom: 8

  //set to between 0 and -250 to move the playmarker to the left, you may have to adjust the zoom acordingly. Does not currently support moving to the right.
  readonly property int waveformPlaymarkerPosition: -250
  
  //0 = RGB, 1 = Default, 2 = Red, 3 = Dark Orange, 4 = Light Orange, 5 = Warm Yellow, 6 = Yellow, 7 = Lime, 8 = Green, 9 = Mint, 10 = Cyan, 11 = Turquoise, 12 = Blue, 13 = Plum, 14 = Violet, 15 = Purple, 16 = Magenta, 17 = Fuchsia
  readonly property int deckAWaveformColour:  10 // origine : 0
  readonly property int deckBWaveformColour:  10 // origine : 0
  readonly property int deckCWaveformColour:  10 // origine : 0
  readonly property int deckDWaveformColour:  10 // origine : 0
  
  //0 = Red, 1 = Dark Orange, 2 = Light Orange, 3 = Default, 4 = Yellow, 5 = Lime, 6 = Green, 7 = Mint, 8 = Cyan, 9 = Turquoise, 10 = Blue, 11 = Plum, 12 = Violet, 13 = Purple, 14 = Magenta, 15 = Fuchsia, 16 = White
  readonly property int phaseAColour:  13
  readonly property int phaseBColour:  13
  readonly property int phaseCColour:  13
  readonly property int phaseDColour:  13
  
  //0 = Red, 1 = Dark Orange, 2 = Light Orange, 3 = White, 4 = Yellow, 5 = Lime, 6 = Green, 7 = Mint, 8 = Cyan, 9 = Turquoise, 10 = Blue, 11 = Plum, 12 = Violet, 13 = Purple, 14 = Magenta, 15 = Fuchsia
  readonly property int deckAColour:  10
  readonly property int deckBColour:  10
  readonly property int deckCColour:  2
  readonly property int deckDColour:  2
  readonly property int jogAColour:  10
  readonly property int jogBColour:  10
  readonly property int jogCColour:  2
  readonly property int jogDColour:  2
  
  //1 = Red, 2 = Dark Orange, 3 = Light Orange, 4 = White, 5 = Yellow, 6 = Lime, 7 = Green, 8 = Mint, 9 = Cyan, 10 = Turquoise, 11 = Blue, 12 = Plum, 13 = Violet, 14 = Purple, 15 = Magenta, 16 = Fuchsia
  readonly property int fx1Colour:  1
  readonly property int fx2Colour:  8
  readonly property int fx3Colour:  11
  readonly property int fx4Colour:  6
  readonly property int filterColour:  3
  
  //change to false to hide the Beats box
  readonly property bool enableBeatsBox: false
  
  //enable to show beats to cue as default in the beats box
  readonly property bool showBeatsToCue: true
  
  //change to true to show elapsed time instead of remaining time. Non selected option is still visible by holding shift.
  readonly property bool elapsedTime:  false
  
  //change to false to show the master bpm instead of the bpm difference when holding shift
  readonly property bool bpmDifference:  true
  
  //change to true to display the shift value for the BPM box by default and the current BPM on shift
  readonly property bool bpmSwap:  false
  
  //when bpmSwap is enabled, the master deck will show it's current bpm instead of '0.0' or '0.00%'
  readonly property bool showMasterBpm: false
  
  //true = show desiredPitch option in the tempo box when holding shift, false = show only current pitch/tempo
  readonly property bool shiftExtraPitchDisplay:    false
  
  //true = show pitch needed to match the BPM to the master, when holding shift. false = show pitch range when holding shift. NOTE: For this option to display, shiftExtraPitchDisplay must be set to true
  readonly property bool desiredPitch:   true
  
  //change to false to disable the flashing when track end warning is enabled
  readonly property bool waveformEndFlashing: false // origine : true
  readonly property bool jogwheelEndFlashing: false // origine : true
  
  //change to true to change the flashing to red and green when in an active loop
  readonly property bool loopActiveRedFlash: false
  
  //change to false to disable tempo faders when holding shift
  readonly property bool shiftTempoFaderEnable: false // origine : true

  //set to true to disable the browser closing when a track is loaded.
  readonly property bool disableKnobFullBrowserCloser: false
  readonly property bool disableKnobSingleBrowserCloser: false
}
